/**
 * @fileoverview XMLインストラクションを使い、左右の柱を生成して適用する
 * @author Kenshi Muto &lt;kmuto@debian.org&gt;
 * @requires libCommon.jsx
 * @version 2,3
*/

/*
    Copyright:
    2008 Kenshi Muto <kmuto@debian.org>
----------------------------------------------------------------------
ソフトウェア使用許諾同意書
本ソフトウェアの利用・変更・再配布にあたっては、下記の使用許諾同意書に
同意する必要があります。

1. 本使用許諾同意書における「ソフトウェア」とは、機械可読の資料 (ライブ
   ラリ、スクリプト、ソースファイル、データファイル)、実行形式、および
   文書を意味します。
2. 本ソフトウェアの使用許諾同意書に同意する限りにおいて、使用者は
   本ソフトウェアを自由に利用、変更することができます。
3. 本ソフトウェアに変更を加えない限りにおいて、使用者は本ソフトウェアを
   自由にコピー、再配布することができます。
4. 本ソフトウェアは無保証です。作者およびそれに関連する組織、配布者は、
   本ソフトウェアの使用に起因する一切の直接損害、間接損害、偶発的損害、
   特別損害、懲戒的損害、派生的損害について何らの責任・保証も負いません。
5. 本ソフトウェアを変更した上で再配布するときには、下記の事項すべてに
   従わなければなりません。
   - 使用許諾同意書の内容に変更を加えてはなりません。技術上の理由で
     文字エンコーディングの変換を行うことは許可しますが、その使用者が
     特殊な技術的措置なしに可読な形でなければなりません。
   - 技術上の理由でバイナリ化・難読化を行う場合も、変更箇所を含めた
     ソフトウェアを、その使用者が可読可能な形式の形で同一のメディアで
     提供しなければなりません。本使用許諾同意書の2条および3条により、
     使用者が可読形式の該当ソフトウェアを変更、コピー、再配布することを
     妨げてはなりません。
   - ソフトウェア構成物の所定の作者名の欄に、変更者のクレジット
     (個人名、企業名、所属、連絡先など)を「追加」しなければなりません。
6. 本ソフトウェアを変更した上で再配布するときには、変更理由および
   その内容を明記することが推奨されます。
7. 使用者がソフトウェアに適用可能な特許に対して特許侵害にかかわる何らか
   の行動を開始した時点で、この使用許諾同意書は自動的に終了し、以降
   使用者はこの使用許諾書によって与えられた一切の権利を放棄するものと
   します。

著作権所有者 Copyright (C) 2008 Kenshi Muto. All rights reserved.
使用許諾同意書バージョン1.0
著作権所有者による書面での事前の許可がない限り、この使用許諾同意書
に変更を加えてはなりません。
----------------------------------------------------------------------
*/
#include "../libs/libCommon.jsx"

/**
 * マスターページのコピーを作成し、左右柱のテンプレート文字列を置換する
 * @param {Document} document ドキュメントオブジェクト
 * @param {Spread} template 元にするマスターページ
 * @param {String} secprefix マスターページのプリフィクス名
 * @param {String} leftname 左柱のテンプレート文字列
 * @param {String} leftcontent 左柱に挿入する文字列
 * @param {String} leftnumstylename 左柱文字列から全角空白文字列が見つかるまで適用する文字スタイル名(空にした場合は適用しない)
 * @param {boolean} leftnumonly 左柱に文字スタイルを適用する際に数値とA-Zだけに適用する
 * @param {String} rightname 右柱のテンプレート文字列
 * @param {String} rightcontent 右柱に挿入する文字列
 * @param {String} rightnumstylename 右柱文字列から全角空白文字列が見つかるまで適用する文字スタイル名(空にした場合は適用しない)
 * @param {boolean} rightnumonly 右柱に文字スタイルを適用する際に数値とA-Zだけに適用する
 * @type Spread
 * @return 生成したマスターページ
*/
function makeMasterPage(document, template, secprefix, leftname, leftcontent, leftnumstylename, leftnumonly, rightname, rightcontent, rightnumstylename, rightnumonly) {
  var spread = template.duplicate(LocationOptions.atEnd, template);
  var secname = rightcontent + "";
  if (secname == "") secname = leftcontent;
  var no = null;
  if ((rightcontent + "").search(/[　\t]/) > -1) {
    var array = rightcontent.split(/[　\t]/,  2);
    no = array[0];
    secname = array[1];
  }
  try {
  spread.baseName = secname + "";
  } catch (e) { // 同じ名前のマスターページがある場合に退避
    spread.baseName = secname + "2";
  }
  spread.namePrefix = secprefix + "";
  
  var tf = findStringFromObject(spread, leftname)[0];
  if (getCSVersion() == 2) {
    tf.search(leftname, false, false, false, false, leftcontent + "");
  } else {
    // XXX: CS3
    app.findTextPreferences = NothingEnum.nothing;
    app.changeTextPreferences = NothingEnum.nothing;
    app.findTextPreferences.findWhat = leftname;
    app.changeTextPreferences.changeTo = leftcontent + "";
    tf.changeText();
  }

  if (tf.contents == leftname) tf.contents = leftcontent + "";
  if (leftnumstylename != null) setCharacterStyle(document, tf, "　", leftnumstylename, leftnumonly);
  tf = findStringFromObject(spread, rightname)[0];
  if (tf.contents == rightname) tf.contents = rightcontent + "";
  //FIXME set true/false
  if (rightnumstylename != null) setCharacterStyle(document, tf, "　", rightnumstylename, rightnumonly);
  
  return spread;
}

/**
 * XMLインストラクションを検索して&lt;?dtp level="数値" section="文字列" ?&gt;形式のものを発見したら柱とする
 * @param {Document} document ドキュメントオブジェクト
 * @param {XMLItems} items XMLItemsオブジェクト
 * @param {String} secprefix マスターページのプリフィクス名
 * @param {Spread} template 元にするマスターページ
 * @param {String} tobira 扉ページのドキュメントオフセット値(このページには柱を適用しない)
 * @param {String} leftname 左柱のテンプレート文字列
 * @param {Long} leftlevel 左柱に使うレベルの深さ
 * @param {String} leftnumstylename 左柱文字列から全角空白文字列が見つかるまで適用する文字スタイル名(空にした場合は適用しない)
 * @param {boolean} leftnumonly 左柱に文字スタイルを適用する際に数値とA-Zだけに適用する
 * @param {String} leftcontent 左柱に挿入する文字列
 * @param {String} rightname 右柱のテンプレート文字列
 * @param {Long} rightlevel 右柱に使うレベルの深さ
 * @param {String} rightnumstylename 右柱文字列から全角空白文字列が見つかるまで適用する文字スタイル名(空にした場合は適用しない)
 * @param {boolean} rightnumonly 右柱に文字スタイルを適用する際に数値とA-Zだけに適用する
 * @param {String} rightcontent 右柱に挿入する文字列
 * @type boolean
 * @return 再帰のために関数処理を継続するか(true=継続する、false=継続しない)
*/
function processXMLforHashira(document, items, secprefix, template, tobira, leftname, leftlevel, leftnumstylename, leftnumonly, leftcontent, rightname, rightlevel, rightnumstylename, rightnumonly, rightcontent) {
  if (items == null || items.length == 0)  {
    return false;
  }
  
  for (var i = 0; i < items.length; i++) {
    if (items[i].xmlInstructions != null && items[i].xmlInstructions.length > 0) {
      for (var i2 = 0; i2 < items[i].xmlInstructions.length; i2++) {
        var e = items[i].xmlInstructions[i2];
        
        if (e.target == "dtp") {
          if (e.data.search(/section=/) != -1 && e.data.search(/level=/) != -1) {
            var sectname = e.data.match(/section="(.*?)"/)[1].replace(/&lt;/g, '<').replace(/&gt;/g, '>').replace(/&amp;/g, '&');
            var level = parseInt(e.data.match(/level="(.*?)"/)[1]);
            if (level <= rightlevel) {
              // 柱の対象
              if (leftlevel == level) {
                leftcontent = sectname;
                if (rightcontent == "") rightcontent = sectname; // 右柱がないときにはとりあえず章名で埋めておく
              } else rightcontent = sectname;
              
              spread = makeMasterPage(document, template, secprefix, leftname, leftcontent, leftnumstylename, leftnumonly, rightname, rightcontent, rightnumstylename, rightnumonly);
              spread.label = "tmp";
              secprefix = secprefix + 1;
			  var page;
	      if (getCSVersion() == 2) {
                page = e.parent.parentStory.characters[e.storyOffset].insertionPoints[0].parentTextFrames[0].parent.documentOffset;
	      } else {
	        // XXX: CS3
                page = e.storyOffset.parentTextFrames[0].parent.documentOffset + 1; // FIXME:documentOffsetのカウントが異なる?
	      }
              var pages = document.pages;
              for (var i3 = page - 1; i3 < pages.length; i3++) {
                if (i3 == tobira - 1) continue;
                pages[i3].appliedMaster = spread; // 位置から後ろ全部にマスター付け
              }
            }
          }
        }
      }
    }
    processXMLforHashira(document, items[i].xmlItems, secprefix, template, tobira, leftname, leftlevel, leftnumstylename, leftnumonly, leftcontent, rightname, rightlevel, rightnumstylename, rightnumonly, rightcontent);
  }
  return true;
}
